package handlers

import (
	"git.xdrm.io/go/aicra/api"
	"git.xdrm.io/go/articles-api/pkg/model"
)

type articleList struct {
	Articles []model.Article
}

type createArticleRequest struct {
	Author uint
	Title  string
	Body   string
}

type voteRequest struct {
	User    uint
	Article uint
}

func (a *App) HandleGetArticlesByAuthor(req byID) (*articleList, api.Error) {
	articles, err := a.Db.GetArticlesByAuthor(req.ID)
	if err != nil {
		return nil, translateStorageError(err, api.ErrorFailure)
	}
	return &articleList{Articles: articles}, api.ErrorSuccess
}

func (a *App) HandleGetAllArticles() (*articleList, api.Error) {
	articles, err := a.Db.GetAllArticles()
	if err != nil {
		return nil, translateStorageError(err, api.ErrorFailure)
	}
	return &articleList{Articles: articles}, api.ErrorSuccess
}

func (a *App) HandleGetArticleByID(req byID) (*model.Article, api.Error) {
	article, err := a.Db.GetArticleByID(req.ID)
	if err != nil {
		return nil, translateStorageError(err, api.ErrorFailure)
	}
	return article, api.ErrorSuccess
}

func (a *App) HandleCreateArticle(param createArticleRequest) (*model.Article, api.Error) {
	article, err := a.Db.CreateArticle(param.Title, param.Body, param.Author)
	if err != nil {
		return nil, translateStorageError(err, api.ErrorFailure)
	}
	return article, api.ErrorSuccess
}

func (a *App) HandleDeleteArticle(req byID) api.Error {
	err := a.Db.DeleteArticle(req.ID)
	if err != nil {
		return translateStorageError(err, api.ErrorFailure)
	}
	return api.ErrorSuccess
}

func (a *App) HandleUpVote(req voteRequest) (*model.Article, api.Error) {
	_, err := a.Db.UpVote(req.User, req.Article)
	if err != nil {
		return nil, translateStorageError(err, api.ErrorFailure)
	}
	// get article
	return a.HandleGetArticleByID(byID{ID: req.Article})
}
func (a *App) HandleDownVote(req voteRequest) (*model.Article, api.Error) {
	_, err := a.Db.DownVote(req.User, req.Article)
	if err != nil {
		return nil, translateStorageError(err, api.ErrorFailure)
	}
	// get article
	return a.HandleGetArticleByID(byID{ID: req.Article})
}
