package handlers

import (
	"git.xdrm.io/go/aicra/api"
	"git.xdrm.io/go/articles-api/pkg/model"
)

type userList struct {
	Users []model.User
}

type createUserRequest struct {
	Username  string
	Firstname string
	Lastname  string
}

type updateUserRequest struct {
	ID        uint
	Username  *string
	Firstname *string
	Lastname  *string
}

func (a *App) HandleGetAllUsers() (*userList, api.Error) {
	users, err := a.Db.GetAllUsers()
	if err != nil {
		return nil, translateStorageError(err, api.ErrorFailure)
	}
	return &userList{Users: users}, api.ErrorSuccess
}

func (a *App) HandleGetUserByID(req byID) (*model.User, api.Error) {
	user, err := a.Db.GetUserByID(req.ID)
	if err != nil {
		return nil, translateStorageError(err, api.ErrorFailure)
	}
	return user, api.ErrorSuccess
}

func (a *App) HandleCreateUser(req createUserRequest) (*model.User, api.Error) {
	user, err := a.Db.CreateUser(req.Username, req.Firstname, req.Lastname)
	if err != nil {
		return nil, translateStorageError(err, api.ErrorFailure)
	}
	return user, api.ErrorSuccess
}

func (a *App) HandleUpdateUser(req updateUserRequest) (*model.User, api.Error) {
	// nothing to update, ignore
	if req.Username == nil && req.Firstname == nil && req.Lastname == nil {
		return a.HandleGetUserByID(byID{req.ID})
	}

	user, err := a.Db.UpdateUser(req.ID, req.Username, req.Firstname, req.Lastname)
	if err != nil {
		return nil, translateStorageError(err, api.ErrorFailure)
	}

	return user, api.ErrorSuccess
}

func (a *App) HandleDeleteUser(req byID) api.Error {
	err := a.Db.DeleteUser(req.ID)
	if err != nil {
		return translateStorageError(err, api.ErrorFailure)
	}
	return api.ErrorSuccess
}
